@@ -6,7 +6,6 @@ for all indexes.
 import hashlib
 import mimetypes
 from wsgiref.handlers import format_date_time
-import py
 import re
 from devpi_common.metadata import splitbasename
 from devpi_common.types import parse_hash_spec
@@ -87,8 +86,8 @@ def key_from_link(keyfs, link, user, index):
 
 
 def unicode_if_bytes(val):
-    if isinstance(val, py.builtin.bytes):
-        val = py.builtin._totext(val)
+    if isinstance(val, bytes):
+        return val.decode('ascii')
     return val
 
 
@@ -98,6 +97,9 @@ class FileStore:
     def __init__(self, keyfs):
         self.keyfs = keyfs
 
+    def __repr__(self):
+        return f"<{self.__class__.__name__} {self.keyfs!r}>"
+
     def maplink(self, link, user, index, project):
         key = key_from_link(self.keyfs, link, user, index)
         entry = FileEntry(key, readonly=False)
@@ -135,11 +137,10 @@ class FileStore:
 
     def store(self, user, index, basename, content_or_file, dir_hash_spec=None):
         # dir_hash_spec is set for toxresult files
-        hash_spec = None
         if dir_hash_spec is None:
             dir_hash_spec = get_default_hash_spec(content_or_file)
-            # prevent hashing twice
-            hash_spec = dir_hash_spec
+        # prevent hashing twice
+        hash_spec = dir_hash_spec
         hashdir_a, hashdir_b = make_splitdir(dir_hash_spec)
         key = self.keyfs.STAGEFILE(
             user=user, index=index,
@@ -211,7 +212,10 @@ class FileEntry(object):
 
     @property
     def hash_value(self):
-        return self.hash_spec.split("=", 1)[1]
+        if self.hash_spec:
+            return self.hash_spec.split("=", 1)[1]
+        else:
+            return self.hash_spec
 
     @property
     def hash_type(self):
