@@ -14,6 +14,7 @@ from .interfaces import IStorageConnection3
 from .keyfs_types import PTypedKey, RelpathInfo, TypedKey
 from .log import threadlog, thread_push_log, thread_pop_log
 from .log import thread_change_log_prefix
+from .model import RootModel
 from .readonly import ensure_deeply_readonly
 from .readonly import get_mutable_deepcopy
 from .readonly import is_deeply_readonly
@@ -24,7 +25,7 @@ import time
 from devpi_common.types import cached_property
 
 
-notset = object()
+absent = object()
 
 
 class MissingFileException(Exception):
@@ -48,7 +49,7 @@ class TxNotificationThread:
             raise RuntimeError(
                 "cannot register handlers after thread has started")
         keyname = getattr(key, "name", key)
-        assert py.builtin._istext(keyname) or py.builtin._isbytes(keyname)
+        assert isinstance(keyname, str)
         self._on_key_change.setdefault(keyname, []).append(subscriber)
 
     def wait_event_serial(self, serial):
@@ -230,9 +231,17 @@ class KeyFS(object):
             cache_size=cache_size)
         self._readonly = readonly
 
-    def get_connection(self, closing=True, write=False):
-        conn = IStorageConnection3(
-            self._storage.get_connection(closing=False, write=write))
+    def __repr__(self):
+        return f"<{self.__class__.__name__} {self.basedir}>"
+
+    def get_connection(self, closing=True, write=False, timeout=30):
+        try:
+            conn = self._storage.get_connection(
+                closing=False, write=write, timeout=timeout)
+        except TypeError:
+            conn = self._storage.get_connection(
+                closing=False, write=write)
+        conn = IStorageConnection3(conn)
         if closing:
             return contextlib.closing(conn)
         return conn
@@ -312,7 +321,7 @@ class KeyFS(object):
         return getattr(self._threadlocal, "tx")
 
     def add_key(self, name, path, type):
-        assert isinstance(path, py.builtin._basestring)
+        assert isinstance(path, str)
         if "{" in path:
             key = PTypedKey(self, path, type, name)
         else:
@@ -366,6 +375,10 @@ class KeyFS(object):
         tx = self.tx
         if tx.write:
             raise RuntimeError("Can only restart a read transaction.")
+        if tx.at_serial == tx.conn.db_read_last_changelog_serial():
+            threadlog.debug(
+                "already at current serial, not restarting transactions")
+            return
         old_prefix = self._tx_prefix()
         tx.restart(write=False)
         thread_change_log_prefix(self._tx_prefix(), old_prefix)
@@ -468,6 +481,54 @@ def iter_relpaths_at(self, typedkeys, at_serial):
                     value=val)
 
 
+class TransactionRootModel(RootModel):
+    def __init__(self, xom):
+        super().__init__(xom)
+        self.model_cache = {}
+
+    def create_user(self, username, password, **kwargs):
+        if username in self.model_cache:
+            assert self.model_cache[username] is None
+        self.model_cache[username] = super().create_user(
+            username, password, **kwargs)
+        return self.model_cache[username]
+
+    def create_stage(self, user, index, type="stage", **kwargs):
+        key = (user.name, index)
+        if key in self.model_cache:
+            assert self.model_cache[key] is None
+        self.model_cache[key] = super().create_stage(
+            user, index, type=type, **kwargs)
+        return self.model_cache[key]
+
+    def delete_user(self, username):
+        if username in self.model_cache:
+            assert self.model_cache[username] is not None
+            del self.model_cache[username]
+        super().delete_user(username)
+
+    def delete_stage(self, username, index):
+        key = (username, index)
+        if key in self.model_cache:
+            assert self.model_cache[key] is not None
+            del self.model_cache[key]
+        super().delete_stage(username, index)
+
+    def get_user(self, name):
+        if name not in self.model_cache:
+            self.model_cache[name] = super().get_user(name)
+        return self.model_cache[name]
+
+    def getstage(self, user, index=None):
+        if index is None:
+            user = user.strip('/')
+            (user, index) = user.split('/')
+        key = (user, index)
+        if key not in self.model_cache:
+            self.model_cache[key] = super().getstage(user, index)
+        return self.model_cache[key]
+
+
 class Transaction(object):
     def __init__(self, keyfs, at_serial=None, write=False):
         self.keyfs = keyfs
@@ -484,6 +545,7 @@ class Transaction(object):
         self.dirty = set()
         self.closed = False
         self.doomed = False
+        self._model = absent
         self._finished_listeners = []
         self._success_listeners = []
 
@@ -492,6 +554,11 @@ class Transaction(object):
         return self.keyfs.get_connection(
             write=self.write, closing=False)
 
+    def get_model(self, xom):
+        if self._model is absent:
+            self._model = TransactionRootModel(xom)
+        return self._model
+
     def iter_relpaths_at(self, typedkeys, at_serial):
         return self.conn.iter_relpaths_at(typedkeys, at_serial)
 
@@ -535,7 +602,7 @@ class Transaction(object):
 
     def get_key_in_transaction(self, relpath):
         for key in self.cache:
-            if key.relpath == relpath:
+            if key.relpath == relpath and self.cache[key] is not absent:
                 return key
         raise KeyError(relpath)
 
@@ -559,14 +626,16 @@ class Transaction(object):
             either as a readonly-view or as a mutable deep copy. """
         try:
             val = self.cache[typedkey]
+            if val is absent:
+                raise KeyError
         except KeyError:
-            absent = typedkey in self.dirty
-            if not absent:
+            absent_from_dirty = typedkey in self.dirty
+            if not absent_from_dirty:
                 try:
                     val = self.get_original(typedkey)
                 except KeyError:
-                    absent = True
-            if absent:
+                    absent_from_dirty = True
+            if absent_from_dirty:
                 # for convenience we return an empty instance
                 # but below we still respect the readonly property
                 val = typedkey.type()
@@ -580,14 +649,16 @@ class Transaction(object):
 
     def exists(self, typedkey):
         if typedkey in self.cache:
-            return True
+            return self.cache[typedkey] is not absent
         if typedkey in self.dirty:
             return False
         try:
             val = self.get_value_at(typedkey, self.at_serial)
         except KeyError:
+            self.cache[typedkey] = absent
             return False
         else:
+            assert val is not absent
             assert is_deeply_readonly(val)
             self.cache[typedkey] = val
             return True
@@ -608,7 +679,7 @@ class Transaction(object):
         try:
             old_val = self.get_original(typedkey)
         except KeyError:
-            old_val = notset
+            old_val = absent
         self.cache[typedkey] = val
         if val != old_val:
             self.dirty.add(typedkey)
@@ -634,10 +705,11 @@ class Transaction(object):
             with self.conn.write_transaction() as fswriter:
                 for typedkey in self.dirty:
                     val = self.cache.get(typedkey)
+                    assert val is not absent
                     # None signals deletion
                     fswriter.record_set(typedkey, val)
-                commit_serial = getattr(fswriter, "commit_serial", notset)
-                if commit_serial is notset:
+                commit_serial = getattr(fswriter, "commit_serial", absent)
+                if commit_serial is absent:
                     # for storages which don't have the attribute yet
                     commit_serial = self.conn.last_changelog_serial + 1
         finally:
@@ -668,6 +740,8 @@ class Transaction(object):
             # ``rollback``, which then arrives here.
             return
         threadlog.debug("closing transaction at %s", self.at_serial)
+        del self._model
+        del self._original
         del self.cache
         del self.dirty
         self.conn.close()
@@ -698,9 +772,9 @@ class Transaction(object):
 
 def check_unicode_keys(d):
     for key, val in d.items():
-        assert not isinstance(key, py.builtin.bytes), repr(key)
+        assert not isinstance(key, bytes), repr(key)
         # not allowing bytes seems ok for now, we might need to relax that
         # it certainly helps to get unicode clean
-        assert not isinstance(val, py.builtin.bytes), repr(key) + "=" + repr(val)
+        assert not isinstance(val, bytes), repr(key) + "=" + repr(val)
         if isinstance(val, dict):
             check_unicode_keys(val)
