@@ -9,11 +9,13 @@ from .mythread import current_thread
 from .readonly import ReadonlyView
 from .readonly import ensure_deeply_readonly, get_mutable_deepcopy
 from .sizeof import gettotalsizeof
+from io import BytesIO
 from repoze.lru import LRUCache
+from tempfile import SpooledTemporaryFile as SpooledTemporaryFileBase
 from zope.interface import implementer
 import contextlib
 import os
-import py
+import shutil
 import sqlite3
 import time
 
@@ -21,6 +23,21 @@ import time
 absent = object()
 
 
+class SpooledTemporaryFile(SpooledTemporaryFileBase):
+    # some missing methods
+    def readable(self):
+        return self._file.readable()
+
+    def readinto(self, buffer):
+        return self._file.readinto(buffer)
+
+    def seekable(self):
+        return self._file.seekable()
+
+    def writable(self):
+        return self._file.writable()
+
+
 class BaseConnection:
     _get_relpath_at = get_relpath_at
 
@@ -53,6 +70,16 @@ class BaseConnection:
         for row in rows:
             print(row)
 
+    def executemany(self, query, *args):
+        c = self._sqlconn.cursor()
+        # print(query)
+        # self._print_rows(self._explain(query, *args))
+        # self._print_rows(self._explain_query_plan(query, *args))
+        r = c.executemany(query, *args)
+        result = r.fetchall()
+        c.close()
+        return result
+
     def fetchall(self, query, *args):
         c = self._sqlconn.cursor()
         # print(query)
@@ -106,9 +133,9 @@ class BaseConnection:
             raise KeyError(relpath)
         return tuple(row[:2])
 
-    def db_write_typedkey(self, relpath, name, next_serial):
+    def db_write_typedkeys(self, keys):
         q = "INSERT OR REPLACE INTO kv (key, keyname, serial) VALUES (?, ?, ?)"
-        self.fetchone(q, (relpath, name, next_serial))
+        self.executemany(q, keys)
 
     def write_changelog_entry(self, serial, entry):
         threadlog.debug("writing changelog for serial %s", serial)
@@ -179,6 +206,9 @@ class BaseConnection:
                     serial=serial, back_serial=back_serial,
                     value=val)
 
+    def write_transaction(self):
+        return Writer(self.storage, self)
+
 
 @implementer(IStorageConnection2)
 class Connection(BaseConnection):
@@ -187,22 +217,58 @@ class Connection(BaseConnection):
 
     def io_file_exists(self, path):
         assert not os.path.isabs(path)
+        f = self.dirty_files.get(path, absent)
+        if f is not absent:
+            return f is not None
         q = "SELECT path FROM files WHERE path = ?"
         result = self.fetchone(q, (path,))
         return result is not None
 
-    def io_file_set(self, path, content):
+    def io_file_set(self, path, content_or_file):
         assert not os.path.isabs(path)
         assert not path.endswith("-tmp")
-        q = "INSERT OR REPLACE INTO files (path, size, data) VALUES (?, ?, ?)"
-        self.fetchone(q, (path, len(content), sqlite3.Binary(content)))
-        self.dirty_files[path] = True
+        f = self.dirty_files.get(path, None)
+        if f is None:
+            f = SpooledTemporaryFile(max_size=1048576)
+        if not isinstance(content_or_file, bytes) and not callable(getattr(content_or_file, "seekable", None)):
+            content_or_file = content_or_file.read()
+            if len(content_or_file) > 1048576:
+                threadlog.warn(
+                    "Read %.1f megabytes into memory in postgresql io_file_set for %s, because of unseekable file",
+                    len(content_or_file) / 1048576, path)
+        if isinstance(content_or_file, bytes):
+            f.write(content_or_file)
+            f.seek(0)
+        else:
+            content_or_file.seek(0)
+            shutil.copyfileobj(content_or_file, f)
+        self.dirty_files[path] = f
 
     def io_file_open(self, path):
-        return py.io.BytesIO(self.io_file_get(path))
+        dirty_file = self.dirty_files.get(path, absent)
+        if dirty_file is None:
+            raise IOError()
+        if dirty_file is absent:
+            return BytesIO(self.io_file_get(path))
+        f = SpooledTemporaryFile()
+        # we need a new file to prevent the dirty_file from being closed
+        dirty_file.seek(0)
+        shutil.copyfileobj(dirty_file, f)
+        dirty_file.seek(0)
+        f.seek(0)
+        return f
 
     def io_file_get(self, path):
         assert not os.path.isabs(path)
+        f = self.dirty_files.get(path, absent)
+        if f is None:
+            raise IOError()
+        elif f is not absent:
+            pos = f.tell()
+            f.seek(0)
+            content = f.read()
+            f.seek(pos)
+            return content
         q = "SELECT data FROM files WHERE path = ?"
         content = self.fetchone(q, (path,))
         if content is None:
@@ -211,6 +277,14 @@ class Connection(BaseConnection):
 
     def io_file_size(self, path):
         assert not os.path.isabs(path)
+        f = self.dirty_files.get(path, absent)
+        if f is None:
+            raise IOError()
+        elif f is not absent:
+            pos = f.tell()
+            size = f.seek(0, 2)
+            f.seek(pos)
+            return size
         q = "SELECT size FROM files WHERE path = ?"
         result = self.fetchone(q, (path,))
         if result is not None:
@@ -218,15 +292,58 @@ class Connection(BaseConnection):
 
     def io_file_delete(self, path):
         assert not os.path.isabs(path)
-        q = "DELETE FROM files WHERE path = ?"
-        self.fetchone(q, (path,))
+        f = self.dirty_files.pop(path, None)
+        if f is not None:
+            f.close()
         self.dirty_files[path] = None
 
-    def write_transaction(self):
-        return Writer(self.storage, self)
+    def _file_write(self, path, f):
+        assert not os.path.isabs(path)
+        assert not path.endswith("-tmp")
+        q = "INSERT OR REPLACE INTO files (path, size, data) VALUES (?, ?, ?)"
+        f.seek(0)
+        content = f.read()
+        f.close()
+        self.fetchone(q, (path, len(content), sqlite3.Binary(content)))
+
+    def _file_delete(self, path):
+        assert not os.path.isabs(path)
+        assert not path.endswith("-tmp")
+        q = "DELETE FROM files WHERE path = ?"
+        self.fetchone(q, (path,))
+
+    def _get_rel_renames(self):
+        return []
+
+    def _write_dirty_files(self, rel_renames):
+        files_del = []
+        files_commit = []
+        for path, f in self.dirty_files.items():
+            if f is None:
+                self._file_delete(path)
+                files_del.append(path)
+            else:
+                self._file_write(path, f)
+                files_commit.append(path)
+        self.dirty_files.clear()
+        return (files_commit, files_del)
+
+    def _drop_dirty_files(self):
+        return
 
     def commit_files_without_increasing_serial(self):
-        self.commit()
+        try:
+            rel_renames = self._get_rel_renames()
+            (files_commit, files_del) = self._write_dirty_files(rel_renames)
+            if files_commit or files_del:
+                threadlog.debug(
+                    "wrote files without increasing serial: %s",
+                    LazyChangesFormatter({}, files_commit, files_del))
+        except BaseException:
+            self.rollback()
+            raise
+        else:
+            self.commit()
 
 
 class BaseStorage(object):
@@ -297,7 +414,7 @@ class BaseStorage(object):
             self._get_sqlconn = self._get_sqlconn_path
             return conn
 
-    def get_connection(self, closing=True, write=False):
+    def get_connection(self, closing=True, write=False, timeout=30):
         # we let the database serialize all writers at connection time
         # to play it very safe (we don't have massive amounts of writes).
         mode = "ro"
@@ -314,14 +431,15 @@ class BaseStorage(object):
                 try:
                     sqlconn.execute("begin immediate")
                     break
-                except sqlite3.OperationalError:
+                except sqlite3.OperationalError as e:
                     # another thread may be writing, give it a chance to finish
-                    time.sleep(0.01)
+                    time.sleep(0.1)
                     if hasattr(thread, "exit_if_shutdown"):
                         thread.exit_if_shutdown()
-                    if time.monotonic() - start_time > 30:
+                    elapsed = time.monotonic() - start_time
+                    if elapsed > timeout:
                         # if it takes this long, something is wrong
-                        raise
+                        raise TimeoutError(f"Timeout after {elapsed} seconds.") from e
         conn = self.Connection(sqlconn, self.basedir, self)
         if closing:
             return contextlib.closing(conn)
@@ -402,7 +520,8 @@ def devpiserver_storage_backend(settings):
     return dict(
         storage=Storage,
         name="sqlite_db_files",
-        description="SQLite backend with files in DB for testing only")
+        description="SQLite backend with files in DB for testing only",
+        hidden=True)
 
 
 @hookimpl
@@ -454,13 +573,22 @@ def devpiserver_metrics(request):
 
 
 class LazyChangesFormatter:
-    __slots__ = ('keys',)
+    __slots__ = ('files_commit', 'files_del', 'keys')
 
-    def __init__(self, changes):
+    def __init__(self, changes, files_commit, files_del):
+        self.files_commit = files_commit
+        self.files_del = files_del
         self.keys = changes.keys()
 
     def __str__(self):
-        return f"keys: {','.join(repr(c) for c in self.keys)}"
+        msg = []
+        if self.keys:
+            msg.append(f"keys: {','.join(repr(c) for c in self.keys)}")
+        if self.files_commit:
+            msg.append(f"files_commit: {','.join(self.files_commit)}")
+        if self.files_del:
+            msg.append(f"files_del: {','.join(self.files_del)}")
+        return ", ".join(msg)
 
 
 class Writer:
@@ -468,38 +596,60 @@ class Writer:
         self.conn = conn
         self.storage = storage
         self.changes = {}
-        self.commit_serial = conn.last_changelog_serial + 1
 
     def record_set(self, typedkey, value=None, back_serial=None):
         """ record setting typedkey to value (None means it's deleted) """
         assert not isinstance(value, ReadonlyView), value
-        if back_serial is None:
-            try:
-                _, back_serial = self.conn.db_read_typedkey(typedkey.relpath)
-            except KeyError:
-                back_serial = -1
-        self.conn.db_write_typedkey(typedkey.relpath, typedkey.name, self.commit_serial)
         # at __exit__ time we write out changes to the _changelog_cache
         # so we protect here against the caller modifying the value later
         value = get_mutable_deepcopy(value)
         self.changes[typedkey.relpath] = (typedkey.name, back_serial, value)
 
     def __enter__(self):
+        self.commit_serial = self.conn.last_changelog_serial + 1
         self.log = thread_push_log("fswriter%s:" % self.commit_serial)
         return self
 
     def __exit__(self, cls, val, tb):
         commit_serial = self.commit_serial
-        thread_pop_log("fswriter%s:" % commit_serial)
-        if cls is None:
-            entry = self.changes, []
-            self.conn.write_changelog_entry(commit_serial, entry)
-            self.conn.commit()
-            self.log.info("committed at %s", commit_serial)
-            self.log.debug(
-                "committed: %s", LazyChangesFormatter(self.changes))
+        try:
+            del self.commit_serial
+            if cls is None:
+                changes_formatter = self.commit(commit_serial)
+                self.log.info("committed at %s", commit_serial)
+                self.log.debug("committed: %s", changes_formatter)
 
-            self.storage._notify_on_commit(commit_serial)
-        else:
-            self.conn.rollback()
-            self.log.info("roll back at %s", commit_serial)
+                self.storage._notify_on_commit(commit_serial)
+            else:
+                self.rollback()
+                self.log.info("roll back at %s", commit_serial)
+        except BaseException:
+            self.rollback()
+            raise
+        finally:
+            thread_pop_log("fswriter%s:" % commit_serial)
+
+    def commit(self, commit_serial):
+        data = []
+        for relpath, (keyname, back_serial, value) in self.changes.items():
+            if back_serial is None:
+                try:
+                    _, back_serial = self.conn.db_read_typedkey(relpath)
+                except KeyError:
+                    back_serial = -1
+                # update back_serial for write_changelog_entry
+                self.changes[relpath] = (keyname, back_serial, value)
+            data.append((relpath, keyname, commit_serial))
+        self.conn.db_write_typedkeys(data)
+        del data
+        rel_renames = self.conn._get_rel_renames()
+        entry = (self.changes, rel_renames)
+        self.conn.write_changelog_entry(commit_serial, entry)
+        (files_commit, files_del) = self.conn._write_dirty_files(rel_renames)
+        self.conn.commit()
+        self.storage.last_commit_timestamp = time.time()
+        return LazyChangesFormatter(self.changes, files_commit, files_del)
+
+    def rollback(self):
+        self.conn._drop_dirty_files()
+        self.conn.rollback()
