@@ -11,17 +11,19 @@ import asyncio
 import py
 import ssl
 import sys
+import threading
 import time
-import traceback
 
 from requests import Response, exceptions
 from requests.utils import DEFAULT_CA_BUNDLE_PATH
 from devpi_common.types import cached_property
 from devpi_common.request import new_requests_session
 from .config import parseoptions, get_pluginmanager
+from .exceptions import lazy_format_exception_only
 from .log import configure_logging, threadlog
 from .log import thread_push_log
 from .model import BaseStage
+from .model import RootModel
 from .views import apireturn
 from . import mythread
 from . import __version__ as server_version
@@ -225,6 +227,7 @@ class XOM:
         self.log = threadlog
         self.polling_replicas = {}
         self._stagecache = {}
+        self._stagecache_lock = threading.Lock()
         if self.is_replica():
             from devpi_server.replica import ReplicaThread
             from devpi_server.replica import register_key_subscribers
@@ -268,16 +271,30 @@ class XOM:
         """ return a per-xom singleton for the given indexpath and key
         or raise KeyError if no such singleton was set yet.
         """
-        return self._stagecache[indexpath][key]
+        with self._stagecache_lock:
+            return self._stagecache[indexpath][key]
 
     def set_singleton(self, indexpath, key, obj):
         """ set the singleton for indexpath/key to obj. """
-        s = self._stagecache.setdefault(indexpath, {})
-        s[key] = obj
+        with self._stagecache_lock:
+            s = self._stagecache.setdefault(indexpath, {})
+            s[key] = obj
+
+    def setdefault_singleton(self, indexpath, key, *, default=None, factory=None):
+        """ get existing singleton, or set the default for indexpath/key to obj. """
+        assert default is None or factory is None
+        with self._stagecache_lock:
+            s = self._stagecache.setdefault(indexpath, {})
+            if key not in s:
+                if default is None:
+                    default = factory()
+                s[key] = default
+            return s[key]
 
     def del_singletons(self, indexpath):
         """ delete all singletones for the given indexpath """
-        self._stagecache.pop(indexpath, None)
+        with self._stagecache_lock:
+            self._stagecache.pop(indexpath, None)
 
     @cached_property
     def supported_features(self):
@@ -288,11 +305,15 @@ class XOM:
             results.update(features)
         return tuple(sorted(results))
 
-    @cached_property
+    @property
     def model(self):
         """ root model object. """
-        from devpi_server.model import RootModel
-        return RootModel(self)
+        try:
+            tx = self.keyfs.tx
+        except AttributeError:
+            return RootModel(self)
+        else:
+            return tx.get_model(self)
 
     def main(self):
         xom = self
@@ -372,15 +393,28 @@ class XOM:
     async def async_httpget(self, url, allow_redirects, timeout=None, extra_headers=None):
         timeout = aiohttp.ClientTimeout(total=timeout)
         connector = aiohttp.TCPConnector(ssl=self._ssl_context)
-        async with aiohttp.ClientSession(timeout=timeout, connector=connector, trust_env=True) as session:
-            async with session.get(
-                url, allow_redirects=allow_redirects, headers=extra_headers
-            ) as response:
-                if response.status < 300:
-                    text = await response.text()
-                else:
-                    text = None
-                return (response, text)
+        try:
+            async with aiohttp.ClientSession(timeout=timeout, connector=connector, trust_env=True) as session:
+                async with session.get(
+                    url, allow_redirects=allow_redirects, headers=extra_headers
+                ) as response:
+                    if response.status < 300:
+                        text = await response.text()
+                    else:
+                        text = None
+                    return (response, text)
+        except OSError as e:
+            location = get_caller_location()
+            threadlog.warn(
+                "OS error during async_httpget of %s at %s: %s",
+                url, location, lazy_format_exception_only(e))
+            return FatalResponse(url, repr(sys.exc_info()[1]))
+        except aiohttp.ClientError as e:
+            location = get_caller_location()
+            threadlog.warn(
+                "OS error during async_httpget of %s at %s: %s",
+                url, location, lazy_format_exception_only(e))
+            return FatalResponse(url, repr(sys.exc_info()[1]))
 
     def httpget(self, url, allow_redirects, timeout=None, extra_headers=None):
         if self.config.offline_mode:
@@ -401,19 +435,19 @@ class XOM:
             location = get_caller_location()
             threadlog.warn(
                 "OS error during httpget of %s at %s: %s",
-                url, location, traceback.format_exception_only(e.__class__, e))
+                url, location, lazy_format_exception_only(e))
             return FatalResponse(url, repr(sys.exc_info()[1]))
         except exceptions.ConnectionError as e:
             location = get_caller_location()
             threadlog.warn(
                 "Connection error during httpget of %s at %s: %s",
-                url, location, traceback.format_exception_only(e.__class__, e))
+                url, location, lazy_format_exception_only(e))
             return FatalResponse(url, repr(sys.exc_info()[1]))
         except self._httpsession.Errors as e:
             location = get_caller_location()
             threadlog.warn(
                 "HTTPError during httpget of %s at %s: %s",
-                url, location, traceback.format_exception_only(e.__class__, e))
+                url, location, lazy_format_exception_only(e))
             return FatalResponse(url, repr(sys.exc_info()[1]))
 
     def view_deriver(self, view, info):
