@@ -323,38 +323,37 @@ class MirrorStage(BaseStage):
         entry.delete()
         (is_expired, links, cache_serial) = self._load_cache_links(project)
         if links is not None:
-            links = ensure_deeply_readonly(
-                list(filter(self._is_file_cached, links)))
-        if not links and cleanup:
+            has_links = any(self._is_file_cached(x) for x in links)
+        else:
+            has_links = False
+        if not has_links and cleanup:
             projects = self.key_projects.get(readonly=False)
             if project in projects:
                 projects.remove(project)
                 self.key_projects.set(projects)
 
+    @property
+    def _list_projects_perstage_lock(self):
+        """ get server wide lock for this index """
+        return self.xom.setdefault_singleton(
+            self.name, "projects_update_lock", factory=threading.Lock)
+
     @property
     def cache_projectnames(self):
         """ cache for full list of projectnames. """
         # we could keep this info inside keyfs but pypi.org
         # produces a several MB list of names and it changes often which
         # would spam the database.
-        try:
-            return self.xom.get_singleton(self.name, "projectnames")
-        except KeyError:
-            cache_projectnames = ProjectNamesCache()
-            self.xom.set_singleton(self.name, "projectnames", cache_projectnames)
-            return cache_projectnames
+        return self.xom.setdefault_singleton(
+            self.name, "projectnames", factory=ProjectNamesCache)
 
     @property
     def cache_retrieve_times(self):
         """ per-xom RAM cache for keeping track when we last updated simplelinks. """
         # we could keep this info in keyfs but it would lead to a write
         # for each remote check.
-        try:
-            return self.xom.get_singleton(self.name, "project_retrieve_times")
-        except KeyError:
-            c = ProjectUpdateCache()
-            self.xom.set_singleton(self.name, "project_retrieve_times", c)
-            return c
+        return self.xom.setdefault_singleton(
+            self.name, "project_retrieve_times", factory=ProjectUpdateCache)
 
     def _get_remote_projects(self):
         headers = {"Accept": SIMPLE_API_ACCEPT}
@@ -383,47 +382,69 @@ class MirrorStage(BaseStage):
         else:
             parser = ProjectHTMLParser(response.url)
             parser.feed(response.text)
-        return (parser.projects, response.headers.get("ETag"))
+        return (
+            {normalize_name(x): x for x in parser.projects},
+            response.headers.get("ETag"))
 
-    def list_projects_perstage(self):
-        """ return set of all projects served through the mirror. """
+    def _stale_list_projects_perstage(self):
+        return {normalize_name(x): x for x in self.key_projects.get()}
+
+    def _update_projects(self):
+        try:
+            (projects, etag) = self._get_remote_projects()
+        except self.UpstreamError as e:
+            threadlog.warn(
+                "upstream error (%s): using stale projects list" % e)
+            return self._stale_list_projects_perstage()
+        old = self.cache_projectnames.get()
+        if not self.cache_projectnames.exists() or old != projects:
+            self.cache_projectnames.set(projects, etag)
+
+            # trigger an initial-load event on master
+            if not self.xom.is_replica():
+                # make sure we are at the current serial
+                # this avoids setting the value again when
+                # called from the notification thread
+                if not self.keyfs.tx.write:
+                    self.keyfs.restart_read_transaction()
+                k = self.keyfs.MIRRORNAMESINIT(user=self.username, index=self.index)
+                # when 0 it is new, when 1 it is pre 6.6.0 with
+                # only normalized names
+                if k.get() in (0, 1):
+                    if not self.keyfs.tx.write:
+                        self.keyfs.restart_as_write_transaction()
+                    k.set(2)
+        else:
+            # mark current without updating contents
+            self.cache_projectnames.mark_current(etag)
+        return projects
+
+    def _list_projects_perstage(self):
+        """ Return the cached project names.
+
+            Only for internal use which makes sure the data isn't modified.
+        """
         if self.offline:
             threadlog.warn("offline mode: using stale projects list")
-            return {normalize_name(x): x for x in self.key_projects.get()}
+            return self._stale_list_projects_perstage()
+        # try without lock first
         if not self.cache_projectnames.is_expired(self.cache_expiry):
             projects = self.cache_projectnames.get()
         else:
-            # no fresh projects or None at all, let's go remote
-            try:
-                (projects, etag) = self._get_remote_projects()
-            except self.UpstreamError as e:
-                threadlog.warn(
-                    "upstream error (%s): using stale projects list" % e)
-                return {normalize_name(x): x for x in self.key_projects.get()}
-            else:
-                old = self.cache_projectnames.get()
-                if not self.cache_projectnames.exists() or old != projects:
-                    self.cache_projectnames.set(projects, etag)
-
-                    # trigger an initial-load event on master
-                    if not self.xom.is_replica():
-                        # make sure we are at the current serial
-                        # this avoids setting the value again when
-                        # called from the notification thread
-                        if not self.keyfs.tx.write:
-                            self.keyfs.restart_read_transaction()
-                        k = self.keyfs.MIRRORNAMESINIT(user=self.username, index=self.index)
-                        # when 0 it is new, when 1 it is pre 6.6.0 with
-                        # only normalized names
-                        if k.get() in (0, 1):
-                            if not self.keyfs.tx.write:
-                                self.keyfs.restart_as_write_transaction()
-                            k.set(2)
+            with self._list_projects_perstage_lock:
+                # retry in case it was updated in another thread
+                if not self.cache_projectnames.is_expired(self.cache_expiry):
+                    projects = self.cache_projectnames.get()
                 else:
-                    # mark current without updating contents
-                    self.cache_projectnames.mark_current(etag)
+                    # no fresh projects or None at all, let's go remote
+                    projects = self._update_projects()
+        return projects
 
-        return {normalize_name(x): x for x in projects}
+    def list_projects_perstage(self):
+        """ Return the project names. """
+        # return a read-only version of the cached data,
+        # so it can't be modified accidentally and we avoid a copy
+        return ensure_deeply_readonly(self._list_projects_perstage())
 
     def is_project_cached(self, project):
         """ return True if we have some cached simpelinks information. """
@@ -739,7 +760,8 @@ class MirrorStage(BaseStage):
         project = normalize_name(project)
         if self.is_project_cached(project):
             return True
-        return project in self.list_projects_perstage()
+        # use the internal method to avoid a copy
+        return project in self._list_projects_perstage()
 
     def list_versions_perstage(self, project):
         try:
@@ -798,21 +820,24 @@ def devpiserver_get_stage_customizer_classes():
 class ProjectNamesCache:
     """ Helper class for maintaining project names from a mirror. """
     def __init__(self):
+        self._lock = threading.RLock()
         self._timestamp = -1
-        self._data = frozenset()
+        self._data = dict()
         self._etag = None
 
     def exists(self):
-        return self._timestamp != -1
+        with self._lock:
+            return self._timestamp != -1
 
     def expire(self):
-        self._timestamp = 0
+        with self._lock:
+            self._timestamp = 0
 
     def is_expired(self, expiry_time):
-        return (time.time() - self._timestamp) >= expiry_time
+        with self._lock:
+            return (time.time() - self._timestamp) >= expiry_time
 
     def get(self):
-        """ Get a copy of the cached data. """
         return self._data
 
     def get_etag(self):
@@ -820,21 +845,26 @@ class ProjectNamesCache:
 
     def add(self, project):
         """ Add project to cache. """
-        self._data = self._data.union({project})
+        with self._lock:
+            self._data[normalize_name(project)] = project
 
     def discard(self, project):
         """ Remove project from cache. """
-        self._data = self._data.difference({project})
+        with self._lock:
+            del self._data[normalize_name(project)]
 
     def set(self, data, etag):
         """ Set data and update timestamp. """
-        if data is not self._data:
-            self._data = frozenset(data)
-        self.mark_current(etag)
+        with self._lock:
+            if data != self._data:
+                assert isinstance(data, dict)
+                self._data = data
+            self.mark_current(etag)
 
     def mark_current(self, etag):
-        self._timestamp = time.time()
-        self._etag = etag
+        with self._lock:
+            self._timestamp = time.time()
+            self._etag = etag
 
 
 class ProjectUpdateInnerLock:
