@@ -1,7 +1,6 @@
 
 import getpass
 import posixpath
-import py
 import re
 import json
 import warnings
@@ -142,6 +141,40 @@ class RootModel:
             threadlog.info("created user %r" % username)
         return user
 
+    def create_stage(self, user, index, type="stage", **kwargs):
+        if index in user.get().get("indexes", {}):
+            raise InvalidIndex("indexname '%s' already exists" % index)
+        if not is_valid_name(index):
+            raise InvalidIndex(
+                "indexname '%s' contains characters that aren't allowed. "
+                "Any ascii symbol besides -.@_ is blocked." % index)
+        stage = user._getstage(index, type, {"type": type})
+        if isinstance(stage.customizer, UnknownCustomizer):
+            raise InvalidIndexconfig("unknown index type %r" % type)
+        # apply default values for new indexes
+        for key, value in stage.get_default_config_items():
+            kwargs.setdefault(key, value)
+        # apply default values from customizer class for new indexes
+        for key, value in stage.customizer.get_default_config_items():
+            kwargs.setdefault(key, value)
+        stage._modify(**kwargs)
+        threadlog.info("created index %s: %s", stage.name, stage.ixconfig)
+        return stage
+
+    def delete_user(self, username):
+        with self.keyfs.USERLIST.update() as userlist:
+            userlist.remove(username)
+
+    def delete_stage(self, username, index):
+        user = self.get_user(username)
+        with user.key.update() as userconfig:
+            indexes = userconfig.get("indexes", {})
+            if index not in indexes:
+                threadlog.info("index %s not exists" % self.index)
+                return False
+            del indexes[index]
+            self.xom.del_singletons(f"{username}/{index}")
+
     def get_user(self, name):
         user = User(self, name)
         if user.key.exists():
@@ -154,21 +187,24 @@ class RootModel:
         return set(user.name for user in self.get_userlist())
 
     def _get_user_and_index(self, user, index=None):
-        if not py.builtin._istext(user):
-            user = user.decode("utf8")
+        assert isinstance(user, str)
         if index is None:
-            user = user.strip("/")
+            assert not user.startswith('/')
+            assert not user.endswith('/')
             user, index = user.split("/")
         else:
-            if not py.builtin._istext(index):
-                index = index.decode("utf8")
+            assert isinstance(index, str)
         return user, index
 
     def getstage(self, user, index=None):
-        username, index = self._get_user_and_index(user, index)
-        user = self.get_user(username)
-        if user is not None:
-            return user.getstage(index)
+        (username, indexname) = self._get_user_and_index(user, index)
+        _user = self.get_user(username)
+        if _user is None:
+            return None
+        ixconfig = _user.get()["indexes"].get(indexname, {})
+        if not ixconfig:
+            return None
+        return _user._getstage(indexname, ixconfig["type"], ixconfig)
 
 
 def ensure_boolean(value):
@@ -248,7 +284,7 @@ def is_valid_name(name):
 
 class InvalidUserconfig(Exception):
     def __init__(self, messages):
-        if isinstance(messages, py.builtin._basestring):
+        if isinstance(messages, str):
             messages = [messages]
         self.messages = messages
         Exception.__init__(self, messages)
@@ -273,7 +309,7 @@ class User:
     visible_keys = ignored_keys.union(info_keys, public_keys)
 
     def __init__(self, parent, name):
-        self.__parent__ = parent
+        self.parent = parent
         self.keyfs = parent.keyfs
         self.xom = parent.xom
         self.name = name
@@ -351,8 +387,7 @@ class User:
             self.getstage(name).delete()
         # delete the user information itself
         self.key.delete()
-        with self.keyfs.USERLIST.update() as userlist:
-            userlist.remove(self.name)
+        self.parent.delete_user(self.name)
 
     def validate(self, password):
         userconfig = self.key.get()
@@ -379,24 +414,7 @@ class User:
         return d
 
     def create_stage(self, index, type="stage", **kwargs):
-        if index in self.get().get("indexes", {}):
-            raise InvalidIndex("indexname '%s' already exists" % index)
-        if not is_valid_name(index):
-            raise InvalidIndex(
-                "indexname '%s' contains characters that aren't allowed. "
-                "Any ascii symbol besides -.@_ is blocked." % index)
-        stage = self._getstage(index, type, {"type": type})
-        if isinstance(stage.customizer, UnknownCustomizer):
-            raise InvalidIndexconfig("unknown index type %r" % type)
-        # apply default values for new indexes
-        for key, value in stage.get_default_config_items():
-            kwargs.setdefault(key, value)
-        # apply default values from customizer class for new indexes
-        for key, value in stage.customizer.get_default_config_items():
-            kwargs.setdefault(key, value)
-        stage._modify(**kwargs)
-        threadlog.info("created index %s: %s", stage.name, stage.ixconfig)
-        return stage
+        return self.parent.create_stage(self, index, type=type, **kwargs)
 
     @cached_property
     def MirrorStage(self):
@@ -416,10 +434,7 @@ class User:
             customizer_cls=customizer_cls)
 
     def getstage(self, indexname):
-        ixconfig = self.get()["indexes"].get(indexname, {})
-        if not ixconfig:
-            return None
-        return self._getstage(indexname, ixconfig["type"], ixconfig)
+        return self.parent.getstage(self.name, indexname)
 
     def getstages(self):
         stages = []
@@ -430,7 +445,7 @@ class User:
 
 class InvalidIndexconfig(Exception):
     def __init__(self, messages):
-        if isinstance(messages, py.builtin._basestring):
+        if isinstance(messages, str):
             messages = [messages]
         self.messages = messages
         Exception.__init__(self, messages)
@@ -637,10 +652,16 @@ class BaseStage(object):
         self.ixconfig = ixconfig
         self.customizer = customizer_cls(self)
         # the following attributes are per-xom singletons
-        self.model = xom.model
         self.keyfs = xom.keyfs
         self.filestore = xom.filestore
 
+    def __repr__(self):
+        return f"<{self.__class__.__name__} {self.name}>"
+
+    @property
+    def model(self):
+        return self.xom.model
+
     def get_indexconfig_from_kwargs(self, **kwargs):
         """Normalizes values and validates keys.
 
@@ -732,12 +753,7 @@ class BaseStage(object):
         return userconfig.get("indexes", {}).get(self.index)
 
     def delete(self):
-        with self.user.key.update() as userconfig:
-            indexes = userconfig.get("indexes", {})
-            if self.index not in indexes:
-                threadlog.info("index %s not exists" % self.index)
-                return False
-            del indexes[self.index]
+        self.model.delete_stage(self.username, self.index)
 
     def key_projsimplelinks(self, project):
         return self.keyfs.PROJSIMPLELINKS(user=self.username,
@@ -809,7 +825,7 @@ class BaseStage(object):
         return frozenset(apply_filter_iter(versions, iterator))
 
     def list_versions(self, project):
-        assert py.builtin._istext(project), "project %r not text" % project
+        assert isinstance(project, str), "project %r not text" % project
         versions = set()
         for stage, res in self.op_sro_check_mirror_whitelist(
                 "list_versions_perstage", project=project):
@@ -867,7 +883,7 @@ class BaseStage(object):
         return max(last_serial, version_serial)
 
     def get_versiondata(self, project, version):
-        assert py.builtin._istext(project), "project %r not text" % project
+        assert isinstance(project, str), "project %r not text" % project
         result = {}
         if not self.filter_versions(project, [version]):
             return result
@@ -1068,7 +1084,7 @@ class BaseStage(object):
                 # If we are currently checking ourself raise the error, it is fatal
                 if stage is self:
                     raise
-                threadlog.warn('Failed to check mirror whitelist. Assume it does not exists (%s)', exc)
+                threadlog.warn('Failed to check mirror whitelist. Assume it does not exist (%s)', exc)
 
     def sro(self):
         """ return stage resolution order. """
@@ -1554,6 +1570,9 @@ class LinkStore:
                 "%s-%s on stage %s at %s",
                 project, version, stage.name, stage.keyfs.tx.at_serial)
 
+    def __repr__(self):
+        return f"<{self.__class__.__name__} {self.project} {self.stage.name} {self.version}>"
+
     @property
     def metadata(self):
         metadata = {}
