@@ -1,6 +1,5 @@
 import contextlib
 import os
-import py
 import re
 import traceback
 from time import time
@@ -230,6 +229,9 @@ def tween_keyfs_transaction(handler, registry):
 
 
 def set_header_devpi_serial(response, tx):
+    if "X-DEVPI-SERIAL" in response.headers:
+        # already set explicitly
+        return
     if tx.commit_serial is not None:
         serial = tx.commit_serial
     else:
@@ -416,12 +418,15 @@ class PyPIView:
         self.context = request.context
         xom = request.registry['xom']
         self.xom = xom
-        self.model = xom.model
         self.log = request.log
 
     @lazy
     def auth(self):
-        return Auth(self.model, self.xom.config.get_auth_secret())
+        return Auth(self.xom, self.xom.config.get_auth_secret())
+
+    @property
+    def model(self):
+        return self.xom.model
 
     def get_auth_status(self):
         identity = self.request.identity
@@ -1192,7 +1197,7 @@ class PyPIView:
                 val = form.get(key, "")
                 if val == "UNKNOWN":
                     val = ""
-                assert py.builtin._istext(val), val
+                assert isinstance(val, str), val
             metadata[key] = val
         return metadata
 
@@ -1658,7 +1663,7 @@ def iter_cache_remote_file(xom, entry, url):
                 threadlog.debug(
                     "put missing file back into place: %s", entry._storepath)
             else:
-                with xom.keyfs._storage.get_connection(write=True) as conn:
+                with xom.keyfs.get_connection(write=True, timeout=300) as conn:
                     conn.io_file_set(entry._storepath, f)
                     # on Windows we need to close the file
                     # before the transaction closes
@@ -1731,7 +1736,7 @@ def iter_remote_file_replica(xom, entry, url):
             entry.tx.conn.io_file_set(entry._storepath, f)
         else:
             # we need a direct write connection to use the io_file_* methods
-            with xom.keyfs._storage.get_connection(write=True) as conn:
+            with xom.keyfs.get_connection(write=True, timeout=300) as conn:
                 conn.io_file_set(entry._storepath, f)
                 # on Windows we need to close the file
                 # before the transaction closes
