@@ -20,6 +20,7 @@ from devpi_server.main import XOM, parseoptions
 from devpi_common.validation import normalize_name
 from devpi_common.url import URL
 from devpi_server.log import threadlog, thread_clear_log
+from io import BytesIO
 from pyramid.authentication import b64encode
 from pyramid.httpexceptions import status_map
 from queue import Queue as BaseQueue
@@ -33,11 +34,14 @@ def server_version():
     return parse_version(__version__)
 
 
-def make_file_url(basename, content, stagename=None, baseurl="http://localhost/"):
+def make_file_url(basename, content, stagename=None, baseurl="http://localhost/", add_hash=True):
     from devpi_server.filestore import get_default_hash_spec, make_splitdir
     hash_spec = get_default_hash_spec(content)
     hashdir = "/".join(make_splitdir(hash_spec))
-    s = "%s{stage}/+f/%s/%s#%s" %(baseurl, hashdir, basename, hash_spec)
+    if add_hash:
+        s = "%s{stage}/+f/%s/%s#%s" % (baseurl, hashdir, basename, hash_spec)
+    else:
+        s = "%s{stage}/+f/%s/%s" % (baseurl, hashdir, basename)
     if stagename is not None:
         s = s.format(stage=stagename)
     return s
@@ -351,7 +355,7 @@ def httpget(pypiurls):
                         xself.url = url
                     xself.allow_redirects = allow_redirects
                     if "content" in fakeresponse:
-                        xself.raw = py.io.BytesIO(fakeresponse["content"])
+                        xself.raw = BytesIO(fakeresponse["content"])
                     xself.headers.setdefault('content-type', fakeresponse.get(
                         'content_type', 'text/html'))
                     if "etag" in fakeresponse:
@@ -550,14 +554,14 @@ class Mapp(MappMixin):
         self.auth = self.testapp.auth = None
 
     def getuserlist(self):
-        r = self.testapp.get("/", {"indexes": False}, {"Accept": "application/json"})
+        r = self.testapp.get_json("/", {"indexes": False})
         assert r.status_code == 200
         return r.json["result"]
 
     def getindexlist(self, user=None):
         if user is None:
             user = self.testapp.auth[0]
-        r = self.testapp.get("/%s" % user, accept="application/json")
+        r = self.testapp.get_json("/%s" % user)
         assert r.status_code == 200
         name = r.json["result"]["username"]
         result = {}
@@ -569,7 +573,7 @@ class Mapp(MappMixin):
         indexname = self._getindexname(indexname)
         if user is None:
             user = self.testapp.auth[0]
-        r = self.testapp.get("/%s" % indexname, accept="application/json")
+        r = self.testapp.get_json("/%s" % indexname)
         assert r.status_code == 200
         return r.json["result"]["projects"]
 
@@ -577,7 +581,7 @@ class Mapp(MappMixin):
         indexname = self._getindexname(indexname)
         if user is None:
             user = self.testapp.auth[0]
-        r = self.testapp.get("/%s/%s" % (indexname, name), accept="application/json")
+        r = self.testapp.get_json("/%s/%s" % (indexname, name))
         assert r.status_code == code
         if r.status_code >= 300:
             return
@@ -777,7 +781,7 @@ class Mapp(MappMixin):
                          name=None, version=None, indexname=None,
                          register=True, code=200, waithooks=False,
                          set_whitelist=True):
-        assert py.builtin._isbytes(content)
+        assert isinstance(content, bytes)
         indexname = self._getindexname(indexname)
         #name_version = splitbasename(basename, checkarch=False)
         #if not name:
@@ -797,6 +801,8 @@ class Mapp(MappMixin):
         # return the file url so users/callers can easily use it
         # (probably the official server response should include the url)
         r.file_url = make_file_url(basename, content, stagename=indexname)
+        r.file_url_no_hash = make_file_url(
+            basename, content, stagename=indexname, add_hash=False)
         return r
 
     def push(self, name, version, index, indexname=None, code=200):
