@@ -1,11 +1,12 @@
 # this file is shared via symlink with devpi-client,
 # so for the time being it must continue to work with Python 2
+from devpi_common.metadata import parse_version
 import pytest
 
 try:
-    from urllib import quote as url_quote
-except ImportError:
     from urllib.parse import quote as url_quote
+except ImportError:
+    from urllib import quote as url_quote  # type: ignore
 
 
 class API:
@@ -123,9 +124,9 @@ class TestIndexThings:
 
     def test_pypi_index_attributes(self, mapp):
         mapp.login_root()
-        data = mapp.getjson("/root/pypi")
+        data = mapp.getjson("/root/pypi?no_projects=")
         res = data["result"]
-        res.pop("projects")
+        res.pop("projects", None)
         assert sorted(res.keys()) == sorted([
             "type", "volatile", "title", "mirror_url", "mirror_web_url_fmt"])
         assert res["type"] == "mirror"
@@ -237,7 +238,6 @@ class TestIndexThings:
         assert res["result"]["description"] == "bar"
 
     def test_whitelist_setting(self, mapp, server_version):
-        from pkg_resources import parse_version
         # pypi_whilelist was removed in 5.0.0
         pypi_whitelist_version = parse_version("5dev")
         mapp.create_and_login_user("cuser7")
@@ -435,7 +435,7 @@ class TestMirrorIndexThings:
         indexconfig = dict(
             type="mirror",
             mirror_url=simpypi.simpleurl,
-            mirror_cache_expiry=1800)
+            mirror_cache_expiry=0)
         mapp.create_index("mirror", indexconfig=indexconfig)
         mapp.use("mirror7/mirror")
         simpypi.add_project('pkg')
@@ -443,8 +443,6 @@ class TestMirrorIndexThings:
         result = mapp.getreleaseslist("pkg")
         assert len(result) == 1
         simpypi.remove_project('pkg')
-        indexconfig['mirror_cache_expiry'] = 0
-        mapp.modify_index("mirror7/mirror", indexconfig=indexconfig)
         result = mapp.getreleaseslist("pkg")
         # serving stale links indefinitely
         # we can't explicitly test for that here, because these tests also run
@@ -474,7 +472,6 @@ class TestMirrorIndexThings:
         assert b'ed7/002b439e9ac84/pkg-1.0.tar.gz' in r.body
 
     def test_releases_urlquoting(self, mapp, server_version, simpypi):
-        from pkg_resources import parse_version
         quoting_devpi_version = parse_version("4.3.1dev")
         if server_version < quoting_devpi_version:
             pytest.skip("devpi-server without mirror url quoting fix")
@@ -494,7 +491,6 @@ class TestMirrorIndexThings:
         assert result[0].endswith('/mirror9/mirror/+e/%s_pkg/pkg-1!2017.4+devpi.zip' % base)
 
     def test_releases_urlquoting_hash(self, mapp, server_version, simpypi):
-        from pkg_resources import parse_version
         quoting_devpi_version = parse_version("4.3.1dev")
         if server_version < quoting_devpi_version:
             pytest.skip("devpi-server without mirror url quoting fix")
