@@ -128,6 +128,17 @@ class TestKeyFS:
         with keyfs.transaction(write=False) as tx:
             assert tx.get(key) == after
 
+    @notransaction
+    def test_not_exists_cached(self, keyfs, monkeypatch):
+        key = keyfs.add_key("NAME", "somekey", dict)
+        with keyfs.transaction() as tx:
+            assert key not in tx.cache
+            assert not tx.exists(key)
+            assert key in tx.cache
+            # make get_value_at fail if it is called
+            monkeypatch.setattr(tx, "get_value_at", lambda k, s: 0 / 0)
+            assert not tx.exists(key)
+
 
 class TestGetKey:
     def test_typed_keys(self, keyfs):
@@ -185,7 +196,7 @@ class Test_addkey_combinations:
 
     def test_addkey_unicode(self, keyfs, type, val):
         pattr = keyfs.add_key("NAME", "hello/{some}", type)
-        attr = pattr(some=py.builtin._totext(b'\xe4', "latin1"))
+        attr = pattr(some=b'\xe4'.decode("latin1"))
         assert not attr.exists()
         assert attr.get() == type()
         assert not attr.exists()
@@ -243,7 +254,7 @@ class TestKey:
         with key1.update() as d:
             with key2.update() as l:
                 l.append(1)
-                d[py.builtin._totext("hello")] = l
+                d["hello"] = l
         assert key1.get()["hello"] == l
 
     def test_get_inplace(self, keyfs):
@@ -598,6 +609,21 @@ class TestTransactionIsolation:
         assert changes1 == changes2
 
 
+@notransaction
+def test_changelog(keyfs):
+    D = keyfs.add_key("NAME", "hello", dict)
+    with keyfs.transaction(write=True):
+        D.set({1: 1})
+    with keyfs.transaction(write=True):
+        D.set({2: 2})
+    with keyfs.transaction(write=False) as tx:
+        changes = list(
+            tx.iter_serial_and_value_backwards(D.relpath, tx.at_serial))
+    assert changes == [
+        (1, {2: 2}),
+        (0, {1: 1})]
+
+
 @notransaction
 class TestDeriveKey:
     def test_direct_from_file(self, keyfs):
