@@ -212,7 +212,7 @@ def test_xom_singleton(xom):
     with pytest.raises(KeyError):
         xom.get_singleton("x/y", "hello")
     xom.set_singleton("x/y", "hello", {})
-    d = {1:2}
+    d = {1: 2}
     xom.set_singleton("x/y", "hello", d)
     d[2] = 3
     assert xom.get_singleton("x/y", "hello") == d
@@ -221,6 +221,18 @@ def test_xom_singleton(xom):
         assert xom.get_singleton("x/y", "hello") is None
 
 
+def test_xom_setdefault_singleton(xom):
+    with pytest.raises(KeyError):
+        xom.get_singleton("x/y", "hello")
+    d1 = xom.setdefault_singleton("x/y", "hello", default={1: 2})
+    assert d1[1] == 2
+    d2 = xom.setdefault_singleton("x/y", "hello", default={1: 3})
+    assert d2[1] == 2
+    assert d1 is d2
+    d3 = xom.setdefault_singleton("x/y", "foo", factory=dict)
+    assert isinstance(d3, dict)
+
+
 @pytest.mark.nomocking
 @pytest.mark.parametrize("url", [
     "http://someserver/path",
