@@ -1,3 +1,4 @@
+import aiohttp
 import requests.exceptions
 import time
 import hashlib
@@ -602,7 +603,10 @@ class TestMirrorStageprojects:
                 <a href='ploy-ansible/'>ploy_ansible</a><br/>
             </body></html>""")
         (projects, etag) = pypistage._get_remote_projects()
-        assert projects == set(["ploy_ansible", "devpi-server", "Django"])
+        assert projects == {
+            "ploy-ansible": "ploy_ansible",
+            "devpi-server": "devpi-server",
+            "django": "Django"}
         s = pypistage.list_projects_perstage()
         assert s == {
             "ploy-ansible": "ploy_ansible",
@@ -621,7 +625,10 @@ class TestMirrorStageprojects:
                     {"name": "ploy_ansible"}
                 ]}""")
         (projects, etag) = pypistage._get_remote_projects()
-        assert projects == set(["ploy_ansible", "devpi-server", "Django"])
+        assert projects == {
+            "ploy-ansible": "ploy_ansible",
+            "devpi-server": "devpi-server",
+            "django": "Django"}
         s = pypistage.list_projects_perstage()
         assert s == {
             "ploy-ansible": "ploy_ansible",
@@ -638,7 +645,7 @@ class TestMirrorStageprojects:
                 <a href='devpi-server'>devpi-server</a><br/>
             </body></html>""")
         (projects, etag) = pypistage._get_remote_projects()
-        assert projects == set(["devpi-server"])
+        assert projects == {"devpi-server": "devpi-server"}
 
     def test_get_remote_projects_etag(self, pypistage):
         orig_etag = '"foo"'
@@ -667,19 +674,28 @@ class TestMirrorStageprojects:
         (projects, etag) = pypistage._get_remote_projects()
         pypistage.cache_projectnames.set(projects, etag)
         assert etag == orig_etag
-        assert projects == set(["ploy_ansible", "devpi-server", "Django"])
+        assert projects == {
+            "ploy-ansible": "ploy_ansible",
+            "devpi-server": "devpi-server",
+            "django": "Django"}
         call = pypistage.xom.httpget.call_log.pop()
         assert 'If-None-Match' not in call['extra_headers']
         (projects, etag) = pypistage._get_remote_projects()
         pypistage.cache_projectnames.set(projects, etag)
         assert etag == orig_etag
-        assert projects == set(["ploy_ansible", "devpi-server", "Django"])
+        assert projects == {
+            "ploy-ansible": "ploy_ansible",
+            "devpi-server": "devpi-server",
+            "django": "Django"}
         call = pypistage.xom.httpget.call_log.pop()
         assert call['extra_headers']['If-None-Match'] == orig_etag
         (projects, etag) = pypistage._get_remote_projects()
         pypistage.cache_projectnames.set(projects, etag)
         assert etag == changed_etag
-        assert projects == set(["ploy", "devpi-server", "Django"])
+        assert projects == {
+            "ploy": "ploy",
+            "devpi-server": "devpi-server",
+            "django": "Django"}
         (call,) = pypistage.xom.httpget.call_log
         assert call['extra_headers']['If-None-Match'] == orig_etag
 
@@ -890,6 +906,14 @@ class TestMirrorStageprojects:
         assert not [x for x in msgs if "/:bar" in x and "mockresponse" not in x]
         assert [x for x in msgs if "/:***" in x]
 
+    @pytest.mark.notransaction
+    def test_del_singletons(self, pypistage):
+        with pypistage.xom.keyfs.transaction(write=True):
+            pypistage.has_project_perstage("pkg")
+            assert pypistage.name in pypistage.xom._stagecache
+            pypistage.delete()
+            assert pypistage.name not in pypistage.xom._stagecache
+
 
 def raise_ValueError():
     raise ValueError(42)
@@ -931,6 +955,24 @@ def test_requests_httpget_error(exc, xom, monkeypatch):
     assert r.status_code == -1
 
 
+@pytest.mark.asyncio
+@pytest.mark.nomocking
+@pytest.mark.parametrize("exc", [
+    OSError,
+    aiohttp.ClientError])
+async def test_async_httpget_error(exc, xom, monkeypatch):
+    from contextlib import asynccontextmanager
+
+    @asynccontextmanager
+    async def async_httpget(self, url, **kw):
+        raise exc()
+        yield
+
+    monkeypatch.setattr(aiohttp.ClientSession, "get", async_httpget)
+    r = await xom.async_httpget("http://notexists.qwe", allow_redirects=False)
+    assert r.status_code == -1
+
+
 def test_is_project_cached(pypistage):
     assert not pypistage.is_project_cached("xyz")
     assert not pypistage.has_project("xyz")
@@ -1007,19 +1049,19 @@ class TestProjectNamesCache:
         return ProjectNamesCache()
 
     def test_get_set(self, cache):
-        assert cache.get() == set()
-        s = set([1, 2, 3])
+        assert cache.get() == dict()
+        s = {1: 1, 2: 2, 3: 3}
         cache.set(s, '"foo"')
-        s.add(4)
-        assert cache.get() != s
-        cache.add(5)
-        assert 5 in cache.get()
-        cache.discard(5)
-        assert 5 not in cache.get()
+        s[4] = 4
+        assert cache.get() == s
+        cache.add('Foo')
+        assert 'foo' in cache.get()
+        cache.discard('foo')
+        assert 'foo' not in cache.get()
 
     def test_is_expired(self, cache, monkeypatch):
         expiry_time = 100
-        s = set([1, 2, 3])
+        s = {1: 1, 2: 2, 3: 3}
         assert cache.get_etag() is None
         cache.set(s, '"foo"')
         assert not cache.is_expired(expiry_time)
