@@ -1,7 +1,6 @@
 import getpass
 import inspect
 import itertools
-import py
 import pytest
 import json
 
@@ -15,7 +14,7 @@ from devpi_server.model import ensure_acl_list
 from devpi_server.model import ensure_list
 from devpi_server.model import run_passwd
 from devpi_server.model import StageCustomizer
-from py.io import BytesIO
+from io import BytesIO
 
 pytestmark = [pytest.mark.writetransaction]
 
@@ -24,7 +23,8 @@ def udict(**kw):
     """ return a dict where the keys are normalized to unicode. """
     d = {}
     for name, val in kw.items():
-        d[py.builtin._totext(name)] = val
+        name = str(name)
+        d[name] = val
     return d
 
 
@@ -40,7 +40,7 @@ def get_release_basenames(stage, project):
 
 
 def register_and_store(stage, basename, content=b"123", name=None):
-    assert py.builtin._isbytes(content), content
+    assert isinstance(content, bytes), content
     n, version = splitbasename(basename)[:2]
     if name is None:
         name = n
@@ -132,7 +132,7 @@ def test_get_mirror_whitelist_info_private_package(mapp, monkeypatch, testapp):
         {"name": "pkg1", "version": "2.6", "description": "foo"},
         set_whitelist=False)
     # test that getting the whitelist info doesn't reveal package name, see #451
-    with mapp.xom.model.keyfs.transaction(write=False):
+    with mapp.xom.keyfs.transaction(write=False):
         with monkeypatch.context() as m:
             # make sure we get an error if data is fetched
             m.setattr(mapp.xom, 'httpget', None)
@@ -145,7 +145,7 @@ def test_get_mirror_whitelist_info_private_package(mapp, monkeypatch, testapp):
         "https://pypi.org/simple/", text='<a href="pkg1"></a>')
     mapp.xom.httpget.mock_simple("pkg1", text="")
     mapp.get_simple("pkg1")
-    with mapp.xom.model.keyfs.transaction(write=False):
+    with mapp.xom.keyfs.transaction(write=False):
         with monkeypatch.context() as m:
             # make sure we get an error if data is fetched
             m.setattr(mapp.xom, 'httpget', None)
@@ -160,7 +160,7 @@ def test_get_mirror_whitelist_info_private_package(mapp, monkeypatch, testapp):
             assert info['blocked_by_mirror_whitelist'] == "root/pypi"
     # now we whitelist the package
     testapp.patch_json("/" + api.stagename, ["mirror_whitelist+=pkg1"])
-    with mapp.xom.model.keyfs.transaction(write=False):
+    with mapp.xom.keyfs.transaction(write=False):
         stage = mapp.xom.model.getstage(api.stagename)
         info = stage.get_mirror_whitelist_info("pkg1")
         assert info['has_mirror_base'] is True
@@ -765,9 +765,8 @@ class TestStage:
     @pytest.mark.notransaction
     def test_mirror_whitelist_inheritance_bbb(self, xom):
         keyfs = xom.keyfs
-        model = xom.model
         with keyfs.transaction(write=True):
-            user = model.create_user("hello", password="123")
+            user = xom.model.create_user("hello", password="123")
             config = udict(index="world", bases=(), type="stage", volatile=True)
             stage = user.create_stage(**config)
             assert stage.ixconfig["mirror_whitelist_inheritance"] == "intersection"
@@ -1131,9 +1130,8 @@ class TestStage:
     @pytest.mark.notransaction
     def test_get_last_change_serial_perstage(self, xom):
         current_serial = xom.keyfs.get_current_serial()
-        model = xom.model
         with xom.keyfs.transaction(write=True):
-            user = model.create_user("hello", password="123")
+            user = xom.model.create_user("hello", password="123")
         assert current_serial == xom.keyfs.get_current_serial() - 1
         current_serial = xom.keyfs.get_current_serial()
         with xom.keyfs.transaction(write=True):
@@ -1565,19 +1563,19 @@ def test_user_set_without_indexes(model):
 
 
 @pytest.mark.notransaction
-def test_setdefault_indexes(xom, model):
+def test_setdefault_indexes(xom):
     from devpi_server.main import set_default_indexes
-    with model.keyfs.transaction(write=True):
+    with xom.keyfs.transaction(write=True):
         set_default_indexes(xom.model)
-    with model.keyfs.transaction(write=False):
-        assert model.getstage("root/pypi").ixconfig["type"] == "mirror"
-    with model.keyfs.transaction(write=False):
-        ixconfig = model.getstage("root/pypi").ixconfig
+    with xom.keyfs.transaction(write=False):
+        assert xom.model.getstage("root/pypi").ixconfig["type"] == "mirror"
+    with xom.keyfs.transaction(write=False):
+        ixconfig = xom.model.getstage("root/pypi").ixconfig
         for key in ixconfig:
-            assert py.builtin._istext(key)
-        userconfig = model.get_user("root").get()
+            assert isinstance(key, str)
+        userconfig = xom.model.get_user("root").get()
         for key in userconfig["indexes"]["pypi"]:
-            assert py.builtin._istext(key)
+            assert isinstance(key, str)
 
 
 @pytest.mark.parametrize("key", ("acl_upload", "acl_toxresult_upload", "mirror_whitelist"))
