@@ -1,3 +1,6 @@
+from collections.abc import Mapping
+from collections.abc import Sequence
+from collections.abc import Set
 from devpi_server.readonly import ensure_deeply_readonly
 from devpi_server.readonly import get_mutable_deepcopy
 from devpi_server.readonly import is_deeply_readonly
@@ -25,12 +28,15 @@ class TestGetMutableDeepcopy:
 
 
 class TestDictReadonlyView:
+    def test_type(self) -> None:
+        assert isinstance(ensure_deeply_readonly({}), Mapping)
+
     def test_nonzero(self) -> None:
         assert not ensure_deeply_readonly({})
-        assert ensure_deeply_readonly({1:2})
+        assert ensure_deeply_readonly({1: 2})
 
     def test_simple(self) -> None:
-        d = {1:2}
+        d = {1: 2}
         r = ensure_deeply_readonly(d)
         assert r[1] == 2
         with pytest.raises(KeyError):
@@ -47,12 +53,20 @@ class TestDictReadonlyView:
             r[1].append(1)
 
     def test_update(self) -> None:
-        d = {1:2}
-        d.update(ensure_deeply_readonly({2:3}))
-        assert d == {1:2, 2:3}
+        d = {1: 2}
+        d.update(ensure_deeply_readonly({2: 3}))
+        assert d == {1: 2, 2: 3}
+
+    def test_values(self) -> None:
+        d = ensure_deeply_readonly({1: []})
+        (value,) = d.values()
+        assert is_deeply_readonly(value)
 
 
 class TestSetReadonlyView:
+    def test_type(self) -> None:
+        assert isinstance(ensure_deeply_readonly(set()), Set)
+
     def test_nonzero(self) -> None:
         assert not ensure_deeply_readonly(set())
         assert ensure_deeply_readonly(set([1]))
@@ -70,14 +84,17 @@ class TestSetReadonlyView:
 
     def test_nogetitem(self) -> None:
         with pytest.raises(TypeError):
-            ensure_deeply_readonly(set([1,2]))[0]
+            ensure_deeply_readonly(set([1, 2]))[0]
 
     def test_iter(self) -> None:
-        l = list(ensure_deeply_readonly(set([1,2])))
-        assert l == [1,2]
+        l = list(ensure_deeply_readonly(set([1, 2])))
+        assert l == [1, 2]
 
 
 class TestListReadonlyView:
+    def test_type(self) -> None:
+        assert isinstance(ensure_deeply_readonly([]), Sequence)
+
     def test_nonzero(self) -> None:
         assert not ensure_deeply_readonly([])
         assert ensure_deeply_readonly([1])
@@ -106,6 +123,9 @@ class TestListReadonlyView:
 
 
 class TestTupleReadonlyView:
+    def test_type(self) -> None:
+        assert isinstance(ensure_deeply_readonly(()), Sequence)
+
     def test_nonzero(self) -> None:
         assert not ensure_deeply_readonly(())
         assert ensure_deeply_readonly((1,))
