@@ -71,6 +71,7 @@ class TestChangelog:
         mapp.create_user("this", password="p")
         latest_serial = self.get_latest_serial(testapp)
         r = reqchangelog(latest_serial)
+        assert r.headers['X-DEVPI-SERIAL'] == str(latest_serial)
         body = b''.join(r.app_iter)
         data = loads(body)
         assert "this" in str(data)
@@ -80,7 +81,7 @@ class TestChangelog:
         mapp.create_user("this", password="p")
         latest_serial = self.get_latest_serial(testapp)
         monkeypatch.setattr(MasterChangelogRequest, "MAX_REPLICA_BLOCK_TIME", 0.01)
-        r = reqchangelog(latest_serial+1)
+        r = reqchangelog(latest_serial + 1)
         assert r.status_code == 202
         assert int(r.headers["X-DEVPI-SERIAL"]) == latest_serial
 
@@ -135,6 +136,7 @@ class TestMultiChangelog:
         latest_serial = self.get_latest_serial(testapp)
         assert latest_serial > 1
         r = reqchangelogs(0)
+        assert int(r.headers['X-DEVPI-SERIAL']) == latest_serial
         body = b''.join(r.app_iter)
         data = loads(body)
         assert isinstance(data, list)
@@ -1000,7 +1002,7 @@ def test_replica_user_auth_before_other_plugins(makexom):
     plugin = Plugin()
     # register both plugins, so the above plugin would normally be called first
     xom = makexom(plugins=[replica, plugin])
-    auth = Auth(xom.model, "qweqwe")
+    auth = Auth(xom, "qweqwe")
     with xom.keyfs.transaction(write=False):
         # because the replica auth has tryfirst our plugin shouldn't be called
         with pytest.raises(HTTPForbidden):
@@ -1155,8 +1157,23 @@ class TestFileReplicationSharedData:
 
         # simulate deleted index
         l = []
+
+        class RootModel:
+            def getstage(self, user, index):
+                l.append((user, index))
+
+        class Transaction:
+            model = None
+
+            def get_model(self, xom):
+                if self.model is None:
+                    self.model = RootModel()
+                return self.model
+
         monkeypatch.setattr(
-            shared_data.xom.model, "getstage", lambda u, i: l.append((u, i)))
+            shared_data.xom.keyfs._threadlocal,
+            "tx", Transaction(), raising=False)
+
         shared_data.on_import_file(None, 1, key, None, -1)
         # it should still be queued to check master and get a 410 for sure
         assert shared_data.queue.qsize() == 1
